HH = 'HH'
VV = 'VV'
VV_VH = 'VV+VH'
HH_HV = 'HH+HV'
DUAL_HH = 'DUAL HH'
DUAL_VV = 'DUAL VV'
DUAL_HV = 'DUAL HV'
DUAL_VH = 'DUAL VH'
HH_3SCAN = 'HH 3SCAN'
HH_4SCAN = 'HH 4SCAN'
HH_5SCAN = 'HH 5SCAN'
QUAD = 'quadrature'
HH_VV = 'HH+VV'
HH_HV_VH_VV = 'HH+HV+VH+VV'
FULL = 'full'
UNKNOWN = 'UNKNOWN'
# NISAR
LH_LV="LH+LV"
RH_RV="RH+RV"
